@@ -56,12 +56,6 @@ DATA_TYPES_SUBTYPES = {
     DATA_TYPES.TEXT: [DATA_SUBTYPES.SHORT, DATA_SUBTYPES.RICH],
 }
 
-
-class ORDER_BY_KEYS:
-    COLUMN = 0
-    ASCENDING_VALUE = 1
-
-
 MODEL_STATUS_TRAINED = "Trained"
 MODEL_STATUS_PREPARING = "Preparing"
 MODEL_STATUS_DATA_ANALYSIS = "Data Analysis"
