@@ -12,7 +12,7 @@ from mindsdb_native.libs.controllers.transaction import (
     LearnTransaction, PredictTransaction
 )
 from mindsdb_native.libs.constants.mindsdb import *
-from mindsdb_native.libs.helpers.general_helpers import check_for_updates
+from mindsdb_native.libs.helpers.general_helpers import check_for_updates, load_lmd, load_hmd
 from mindsdb_native.libs.helpers.locking import MDBLock
 from mindsdb_native.libs.helpers.stats_helpers import sample_data
 
@@ -53,6 +53,39 @@ def _prepare_sample_settings(user_provided_settings,
 
     return sample_settings, sample_function
 
+def _prepare_timeseries_settings(user_provided_settings):
+    timeseries_settings = dict(
+        is_timeseries=False
+        ,group_by=None
+        ,order_by=None
+        ,window=None
+        ,dynamic_window=None
+        ,use_previous_target=True
+        ,keep_order_column=True
+        ,nr_predictions=1
+    )
+
+    if len(user_provided_settings) > 0:
+        if 'order_by' not in user_provided_settings:
+            raise Exception('Invalid timeseries settings, please provide `order_by` key [a list of columns]')
+
+        elif 'window' not in user_provided_settings and 'dynamic_window' not in user_provided_settings:
+            raise Exception(f'Invalid timeseries settings, you must specify a window size with either `window` or `dynamic_window` key')
+
+        elif 'window' in user_provided_settings and 'dynamic_window' in user_provided_settings:
+            raise Exception(f'Invalid timeseries settings, you must specify a window size with *EITHER* `window` or `dynamic_window` key, not both!')
+        else:
+            timeseries_settings['is_timeseries'] = True
+
+
+    for k in user_provided_settings:
+        if k in timeseries_settings:
+            timeseries_settings[k] = user_provided_settings[k]
+        else:
+            raise Exception(f'Invalid timeseries setting: {k}')
+
+    return timeseries_settings
+
 
 class Predictor:
     def __init__(self, name, log_level=CONFIG.DEFAULT_LOG_LEVEL):
@@ -72,7 +105,6 @@ class Predictor:
         if CONFIG.CHECK_FOR_UPDATES:
             check_for_updates()
 
-
         if not CONFIG.SAGEMAKER:
             # If storage path is not writable, raise an exception as this can no longer be
             if not os.access(CONFIG.MINDSDB_STORAGE_PATH, os.W_OK):
@@ -90,9 +122,7 @@ class Predictor:
     def learn(self,
               to_predict,
               from_data,
-              group_by=None,    # @TODO: Move this logic to datasource
-              window_size=None, # @TODO: Move this logic to datasource
-              order_by=None,    # @TODO: Move this logic to datasource
+              timeseries_settings=None,
               ignore_columns=None,
               stop_training_in_x_seconds=None,
               backend='lightwood',
@@ -110,9 +140,7 @@ class Predictor:
         :param from_data: the data that you want to learn from, this can be either a file, a pandas data frame, or url or a mindsdb data source
 
         Optional Time series arguments:
-        :param order_by: this order by defines the time series, it can be a list. By default it sorts each sort by column in ascending manner, if you want to change this pass a touple ('column_name', 'boolean_for_ascending <default=true>')
-        :param group_by: This argument tells the time series that it should learn by grouping rows by a given id
-        :param window_size: The number of samples to learn from in the time series
+        :param timeseries_settings: dictionary of options for handling the data as a timeseries
 
         Optional data transformation arguments:
         :param ignore_columns: mindsdb will ignore this column
@@ -128,30 +156,18 @@ class Predictor:
 
         :return:
         """
-        
+
         with MDBLock('exclusive', 'learn_' + self.name):
+            ignore_columns = [] if ignore_columns is None else ignore_columns
+            timeseries_settings = {} if timeseries_settings is None else timeseries_settings
+            advanced_args = {} if advanced_args is None else advanced_args
 
-            if ignore_columns is None:
-                ignore_columns = []
-
-            if group_by is None:
-                group_by = []
-
-            if order_by is None:
-                order_by = []
-
-            # lets turn into lists: predict, ignore, group_by, order_by
             predict_columns = to_predict if isinstance(to_predict, list) else [to_predict]
             ignore_columns = ignore_columns if isinstance(ignore_columns, list) else [ignore_columns]
-            group_by = group_by if isinstance(group_by, list) else [group_by]
-            order_by = order_by if isinstance(order_by, list) else [order_by]
-
-            # lets turn order by into list of tuples if not already
-            # each element ('column_name', 'boolean_for_ascending <default=true>')
-            order_by = [col_name if isinstance(col_name, tuple) else (col_name, True) for col_name in order_by]
-
-            if advanced_args is None:
-                advanced_args = {}
+            if len(predict_columns) == 0:
+                error = 'You need to specify the column[s] you want to predict via the `to_predict` argument!'
+                self.log.error(error)
+                raise ValueError(error)
 
             from_ds = getDS(from_data)
 
@@ -169,16 +185,11 @@ class Predictor:
                 sample_for_training
             )
 
+            timeseries_settings = _prepare_timeseries_settings(timeseries_settings)
+
             self.log.warning(f'Sample for analysis: {sample_for_analysis}')
             self.log.warning(f'Sample for training: {sample_for_training}')
 
-            if len(predict_columns) == 0:
-                error = 'You need to specify a column to predict'
-                self.log.error(error)
-                raise ValueError(error)
-
-            is_time_series = True if len(order_by) > 0 else False
-
             """
             We don't implement "name" as a concept in mindsdbd data sources, this is only available for files,
             the server doesn't handle non-file data sources at the moment, so this shouldn't prove an issue,
@@ -191,7 +202,8 @@ class Predictor:
                 from_data=from_ds,
                 predictions= None,
                 model_backend= backend,
-                sample_function=sample_function
+                sample_function=sample_function,
+                from_data_type=type(from_ds)
             )
 
             light_transaction_metadata = dict(
@@ -200,12 +212,9 @@ class Predictor:
                 data_preparation = {},
                 predict_columns = predict_columns,
                 model_columns_map = from_ds._col_map,
-                model_group_by = group_by,
-                model_order_by = order_by,
-                model_is_time_series = is_time_series,
+                tss=timeseries_settings,
                 data_source = data_source_name,
                 type = transaction_type,
-                window_size = window_size,
                 sample_settings = sample_settings,
                 stop_training_in_x_seconds = stop_training_in_x_seconds,
                 rebuild_model = rebuild_model,
@@ -237,7 +246,8 @@ class Predictor:
                 tags_delimiter = advanced_args.get('tags_delimiter', ','),
                 force_predict = advanced_args.get('force_predict', False),
 
-                breakpoint=self.breakpoint
+                breakpoint = self.breakpoint,
+                setup_args = from_data.setup_args if hasattr(from_data, 'setup_args') else None
             )
 
             if rebuild_model is False:
@@ -247,11 +257,17 @@ class Predictor:
                 old_hmd = {}
                 for k in heavy_transaction_metadata: old_hmd[k] = heavy_transaction_metadata[k]
 
-                with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, light_transaction_metadata['name'] + '_light_model_metadata.pickle'), 'rb') as fp:
-                    light_transaction_metadata = pickle.load(fp)
+                light_transaction_metadata = load_lmd(os.path.join(
+                    CONFIG.MINDSDB_STORAGE_PATH,
+                    light_transaction_metadata['name'],
+                    'light_model_metadata.pickle'
+                ))
 
-                with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, heavy_transaction_metadata['name'] + '_heavy_model_metadata.pickle'), 'rb') as fp:
-                    heavy_transaction_metadata= pickle.load(fp)
+                heavy_transaction_metadata = load_hmd(os.path.join(
+                    CONFIG.MINDSDB_STORAGE_PATH,
+                    heavy_transaction_metadata['name'],
+                    'heavy_model_metadata.pickle'
+                ))
 
                 for k in ['data_preparation', 'rebuild_model', 'data_source', 'type', 'columns_to_ignore', 'sample_margin_of_error', 'sample_confidence_level', 'stop_training_in_x_seconds']:
                     if old_lmd[k] is not None: light_transaction_metadata[k] = old_lmd[k]
@@ -282,8 +298,7 @@ class Predictor:
 
             predictions = self.predict(when_data=when_data, **predict_args)
 
-            with open(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, f'{self.name}_light_model_metadata.pickle'), 'rb') as fp:
-                lmd = pickle.load(fp)
+            lmd = load_lmd(os.path.join(CONFIG.MINDSDB_STORAGE_PATH, self.name, 'light_model_metadata.pickle'))
 
             accuracy_dict = {}
             for col in lmd['predict_columns']:
@@ -344,7 +359,6 @@ class Predictor:
                 _, _, disable_lightwood_transform_cache = _get_memory_optimizations(when_ds.df)
             heavy_transaction_metadata['when'] = when
             heavy_transaction_metadata['name'] = self.name
-            heavy_transaction_metadata['icp'] = self.transaction.hmd['icp']
 
             if backend is not None:
                 heavy_transaction_metadata['model_backend'] = backend
