@@ -8,7 +8,11 @@ from mindsdb_native.libs.data_types.data_source import DataSource
 
 class PostgresDS(DataSource):
 
-    def _setup(self, query=None, host='localhost', user='postgres', password='', database='postgres', port=5432, table=None):
+    def _setup(self, table=None, query=None, database='postgres', host='localhost',
+               port=5432, user='postgres', password=''):
+
+        self._database_name = database
+        self._table_name = table
 
         if query is None:
             query = f'SELECT * FROM {table}'
@@ -30,3 +34,10 @@ class PostgresDS(DataSource):
             col_map[col] = col
 
         return df, col_map
+
+    def name(self):
+        return '{}: {}/{}'.format(
+            self.__class__.__name__,
+            self._database_name,
+            self._table_name
+        )
\ No newline at end of file
